// ############################################
// Definitions of ecliptic coordinate functions
// ############################################

#include "Equatorial.h"

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Mercury.
// t is Julian day number. r in AU.
// ===================================================================
void MercuryEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  MercuryEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Venus.
// t is Julian day number. r in AU.
// ===================================================================
void VenusEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  VenusEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ==============================================================
// Function to calculate geocentric equatorial coordinates of Sun.
// t is Julian day number. r in AU.
// ==============================================================
void SunEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  SunEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Mars.
// t is Julian day number. r in AU.
// ===================================================================
void MarsEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  MarsEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Jupiter.
// t is Julian day number. r in AU.
// ===================================================================
void JupiterEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  JupiterEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Saturn.
// t is Julian day number. r in AU.
// ===================================================================
void SaturnEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  SaturnEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Uranus.
// t is Julian day number. r in AU.
// ===================================================================
void UranusEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  UranusEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===================================================================
// Function to calculate geocentric equatorial coordinates of Neptune.
// t is Julian day number. r in AU.
// ===================================================================
void NeptuneEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  NeptuneEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic 
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}

// ===============================================================
// Function to calculate geocentric equatorial coordinates of Moon.
// t is Julian day number. r in AU.
// ===============================================================
void MoonEquatorial (double t, double& r, double& delta, double& RA, int FK5, int Precess, int Nutate, int Aberrate)
{
  // Get ecliptic coordinates
  double beta, lambda;
  MoonEcliptic (t, r, beta, lambda, FK5, Precess, Nutate, Aberrate);

  // Calculate obliquity of ecliptic
  double eps;
  Obliquity (t, eps);

  // Return equatorial coordinates
  double sinb = sin (beta),    cosb = cos (beta);
  double sinl = sin (lambda),  cosl = cos (lambda);
  double sine = sin (eps),     cose = cos (eps);

  double sind = sinb * cose + cosb * sine * sinl;
  double sinr = cosb * cose * sinl - sinb * sine;
  double cosr = cosb * cosl;

  delta = asin (sind);
  RA    = atan2 (sinr, cosr);
}
